import { renderToStaticMarkup } from "react-dom/server";
import React from "react";

/**
 * 点位聚合
 * @class 用来解决加载大量点要素到地图上产生覆盖现象的问题，并提高性能
 * @constructor
 * @param {Map} map 地图的一个实例。
 * @param {Json Object} options 可选参数，可选项包括：<br />
 *    markers {Array<Marker>} 要聚合的标记数组<br />
 *    gridSize {Number} 聚合计算时网格的像素大小，默认60<br />
 *    maxZoom {Number} 最大的聚合级别，大于该级别就不进行相应的聚合<br />
 *    minClusterSize {Number} 最小的聚合数量，小于该数量的不能成为一个聚合，默认为2<br />
 *    isAverangeCenter {Boolean} 聚合点的落脚位置是否是所有聚合在内点的平均值，默认为否，落脚在聚合内的第一个点<br />
 *    styles {Array<IconStyle>} 自定义聚合后的图标风格，请参考TextIconOverlay类<br />
 */
export const creatMarkerClusterer = (map, opts) => {
  //@ts-ignore
  let markerClusterer = new BMapGLLib.MarkerClusterer(map, {
    maxZoom: 18,
    ...opts,
  });
  map?.setViewport(opts.markers.map((v) => v.getPosition()));
  return markerClusterer;
};

/* marker 创建 extData:存储自定义数据*/
export const creatMarker = (point, opts) => {
  let marker = new BMapGL.Marker(point, opts);
  marker.getExtData = () => opts?.extData || {}; //创建获取存储数据函数
  return marker;
};

/**
 * 创建自定义标注
 * createDOM:创建自定义DOM
 * extData:存储自定义数据
 * 百度文档地址：https://lbsyun.baidu.com/index.php?title=jspopularGL/guide/CustomOverlay
 */
export const creatCustomOverlay = (createDOM, opts) => {
  const { extData, ...reset } = opts;
  //@ts-ignore
  let overlay = new BMapGL.CustomOverlay(createDOM, {
    opacity: 1,
    offsetY: -10,
    properties: opts.extData, //自绑定属性
    ...reset,
  });
  return overlay;
};

/**
 * jsx转换成string
 * @param feature
 * @returns
 */
export const createDOM = (html) => {
  const dom = document.createElement("div");
  dom.innerHTML = renderToStaticMarkup(html);
  return dom;
};

/* Label 创建 */
export const creatLabel = (content, opts) => {
  const { style, ...rest } = opts;
  let Label = new BMapGL.Label(content, { ...rest });
  if (opts.style) {
    Label.setStyle({
      backgroundColor: "transparent",
      border: 0,
      ...opts.style,
    });
  }
  return Label;
};

/* 线条 创建 */
export const creatPolyline = (point, opts) => {
  let polyline = new BMapGL.Polyline(point, {
    strokeColor: "#17F1FF",
    strokeWeight: 2,
    //@ts-ignore
    strokeLineCap: "butt",
    ...opts,
  });
  polyline.getExtData = () => opts?.extData || {}; //创建获取存储数据函数
  return polyline;
};

/* 面创建 */
export const creatPolygon = (point, opts) => {
  let polygon = new BMapGL.Polygon(point, {
    strokeColor: "#FFF71F",
    strokeWeight: 1,
    fillColor: "#FFF71F",
    fillOpacity: 0.1,
    ...opts,
  });
  polygon.getExtData = () => opts?.extData || {}; //创建获取存储数据函数
  return polygon;
};

/**
 * 街况颜色 备注：此方法只适用于街况感知板块道路颜色设置
 * @param val 数值
 * @param type 当前图层类型
 * @returns
 */
export const colorFun = (val, type) => {
  const getColor = (val, ranges) => {
    for (let range of ranges) {
      //@ts-ignore
      if (val <= range.max) {
        return {
          color: range.color,
          opacity: range.opacity,
          icon: range.icon,
          zIndex: range.zIndex,
        };
      }
    }
    // 默认返回最后一个范围的颜色和不透明度
    return ranges[ranges.length - 1];
  };

  //备注：icon是结合css使用，赋值的是css的变量名称
  const problemRanges = [
    {
      max: 3,
      color: "#717171",
      opacity: 0.3,
      icon: "var(--defIcon)",
      zIndex: 1,
    },
    {
      max: 7,
      color: "#AB8130",
      opacity: 0.7,
      icon: "var(--wtIcon2)",
      zIndex: 2,
    },
    {
      max: 12,
      color: "#FFC045",
      opacity: 1,
      icon: "var(--wtIcon3)",
      zIndex: 3,
    },
    {
      max: Infinity,
      color: "#FF5F18",
      opacity: 1,
      icon: "var(--wtIcon4)",
      zIndex: 4,
    },
  ];

  const coverageRanges = [
    {
      max: 0,
      color: "#717171",
      opacity: 0.3,
      icon: "var(--defIcon)",
      zIndex: 1,
    },
    {
      max: 20,
      color: "#5BC0C7",
      opacity: 0.7,
      icon: "var(--fgIcon2)",
      zIndex: 2,
    },
    {
      max: Infinity,
      color: "#17F1FF",
      opacity: 1,
      icon: "var(--fgIcon3)",
      zIndex: 3,
    },
  ];

  return type === "problem"
    ? getColor(val, problemRanges)
    : getColor(val, coverageRanges);
};
